lexer grammar FScript;
options {
  language=Java;

}
@members {
    public void reportError(RecognitionException e) {
    // API does not allow to throw e from here, throwing RuntimeException instead
        throw new RuntimeException("Lexing recognition problem", e);
    }
}
@header {
/*
 * Copyright (c) 2007-2008 ARMINES
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 */
package org.objectweb.fractal.fscript.parser;
}

T14 : 'function' ;
T15 : 'action' ;
T16 : '(' ;
T17 : ')' ;
T18 : ',' ;
T19 : '{' ;
T20 : '}' ;
T21 : 'var' ;
T22 : '=' ;
T23 : ';' ;
T24 : 'return' ;
T25 : 'if' ;
T26 : 'else' ;
T27 : 'for' ;
T28 : ':' ;
T29 : '||' ;
T30 : '&&' ;
T31 : '==' ;
T32 : '!=' ;
T33 : '<' ;
T34 : '<=' ;
T35 : '>' ;
T36 : '>=' ;
T37 : '+' ;
T38 : '-' ;
T39 : '*' ;
T40 : 'div' ;
T41 : '|' ;
T42 : '&' ;
T43 : '\\' ;
T44 : '/' ;
T45 : '$' ;
T46 : '::' ;
T47 : '@' ;
T48 : '..' ;
T49 : '[' ;
T50 : ']' ;
T51 : '.' ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 166
/**
 * Identifiers. Used for variables, procedures and axes names.
 */
ID
    :   LETTER (LETTER | DIGIT | '-')*
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 173
/**
 * Literal numbers.
 */
NUMBER
    :   DIGIT+ ( '.' DIGIT* )?
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 180
/**
 * Literal strings: double or single-quoted with support for standard
 * escape characters.
 */
STRING
    :   DQ_STRING | SQ_STRING
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 188
fragment
DIGIT
    :   '0'..'9'
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 193
fragment
LETTER
    : '\u0041'..'\u005a'
    | '\u005f'
    | '\u0061'..'\u007a'
    | '\u00c0'..'\u00d6'
    | '\u00d8'..'\u00f6'
    | '\u00f8'..'\u00ff'
    | '\u0100'..'\u1fff'
    | '\u3040'..'\u318f'
    | '\u3300'..'\u337f'
    | '\u3400'..'\u3d2d'
    | '\u4e00'..'\u9fff'
    | '\uf900'..'\ufaff'
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 209
/**
 * Double-quoted literal strings.
 */
fragment
DQ_STRING
    :   ('"'  (ESC | ~('\\'|'"'|'\n'))* '"' )
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 217
/**
 * Single-quoted literal strings.
 */
fragment
SQ_STRING
    :   ('\'' (ESC | ~('\\'|'\''|'\n'))* '\'')
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 225
/**
 * Supported escape sequence in literal strings.
 */
ESC
    :   '\\' ('n' | 'r' | 't' | '\'' | '"')
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 232
/**
 * Whitespace: ignored.
 */
WS
    : (' '|'\t'|'\r'|'\n')+ { skip(); }
    ;

// $ANTLR src "/home/chris/adam/fscript/fscript-trunk/target/checkout/fscript/src/main/antlr/org/objectweb/fractal/fscript/parser/FScript.g" 239
/**
 * Single-line comments: ignored.
 */
SL_COMMENT
    :   '--' (~('\n'|'\r'))* ('\n'|'\r'('\n')?) { skip(); }
    ;

